
// NOTICE: 

//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


///////////////////////////////////////////////////////////////////////////
// HEADERS
///////////////////////////////////////////////////////////////////////////


#define VDEXT_VIDEO_FILTER
#define VDEXT_MAIN


#include <stdio.h>
#include <windows.h>
#include <commctrl.h>
#include <commdlg.h>
#include <vfw.h>

#include "resource.h"
#include "Filter.h"


//#include "..\include\StdAfx.h"
#include "..\include\ScriptInterpreter.h"
#include "..\include\ScriptError.h"
#include "..\include\ScriptValue.h"
#include "..\include\VBitmap.h"



///////////////////////////////////////////////////////////////////////////
// FUNCTION DEFS
///////////////////////////////////////////////////////////////////////////

int ezMaskRunProc(const FilterActivation *fa, const FilterFunctions *ff);
int ezMaskStartProc(FilterActivation *fa, const FilterFunctions *ff);
int ezMaskEndProc(FilterActivation *fa, const FilterFunctions *ff);
void deinitProc(FilterActivation *fa, const FilterFunctions *ff);

long ezMaskParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ezMaskConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
void ezMaskStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ezMaskScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool ezMaskFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);
void xRectFill(const FilterActivation *fa, PixCoord x, PixCoord y, PixDim dx, PixDim dy, Pixel32 c);
static const char *openFileDlg(HWND hwnd, const char *oldfn);


LRESULT CALLBACK nPreviewWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);


///////////////////////////////////////////////////////////////////////////
// DATA DEFS
///////////////////////////////////////////////////////////////////////////

//****************************************
// Structure where we hold our Global 
// filter specific Data, like dialog settings,
// and lookup tables.
//
// This is passed into every function ....
//****************************************
typedef struct MyFilterData {
    bool     fExpand;
    bool     fThird;

	RECT rcMask;
	POINTS ptsMove;

	int dstW;
	int dstH;
	long frame;
	long saveFrame;
	int mode;
	int inverse;
	int drawing;
	char szAviPath[MAX_PATH];

	HWND fpHwnd;
	HWND dlgHwnd;
	WNDPROC fpWndProc;
	IFilterPreview *ifp;
	FilterStateInfo *fsi;
    PAVIFILE pfile; 
    PAVISTREAM ppavi; 
	PGETFRAME pgf;
	LPBYTE pData;
	VBitmap *mySrc;

} MyFilterData;

MyFilterData *g_mfd;


void testAVIFile(char * filename, MyFilterData *mfd);

//****************************************
// ?????
//****************************************
ScriptFunctionDef ezMask_func_defs[]={
    { (ScriptFunctionPtr)ezMaskScriptConfig, "Config", "0iiii" },
    { NULL },
};


//****************************************
// ?????
//****************************************
CScriptObject ezMask_obj={
    NULL, ezMask_func_defs
};

//****************************************
// Structure that tells vDub about our filter
// Holds name, description, and pointers to
// the functions vdub calls for the filter
// to do it's thing.
//****************************************
struct FilterDefinition filterDef_ezMask = {

    NULL, NULL, NULL,       // next, prev, module
    "videoOverlay - Alpha",             // name
    "dang",
                            // desc
    "trevlac",               // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size

    NULL,                   // initProc
    deinitProc,        // deinitProc
    ezMaskRunProc,        // runProc
    ezMaskParamProc,      // paramProc
    ezMaskConfigProc,     // configProc
    ezMaskStringProc,     // stringProc
    ezMaskStartProc,      // startProc
    ezMaskEndProc,        // endProc

    &ezMask_obj,          // script_obj
    ezMaskFssProc,        // fssProc

};

static FilterDefinition *fd_ezMask;


///////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////


//****************************************
// VDub calls this exported function to 
// Add our filter to the list ..
//****************************************

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_ezMask = ff->addFilter(fm, &filterDef_ezMask, sizeof(FilterDefinition))))
        return 1;

	INITIALIZE_VTBLS;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

//****************************************
// Exported function to remove us from the list
//****************************************
void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_ezMask);
}


//****************************************
// StartProc - Just before we start to filter
//
// Good place to .....
//****************************************
int ezMaskStartProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if(mfd->ppavi)
		mfd->pgf = AVIStreamGetFrameOpen(mfd->ppavi,NULL);

	mfd->mySrc = new VBitmap();


    return 0;
}


//****************************************
// RunProc gets called for each frame.
//
// This is where the filtering is done.
//****************************************

int ezMaskRunProc(const FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;
    PixDim w, h;
    Pixel32 *src, *dst;

	LPBITMAPINFOHEADER lpbih;
	

	RECT *rcMask, rcDraw;

	rcMask = &(mfd->rcMask);

	mfd->saveFrame = fa->pfsi->lCurrentFrame;

    src = (Pixel32 *)fa->src.data;
    dst = (Pixel32 *)fa->dst.data;

	if(mfd->drawing) {

			h = fa->src.h;
			do {
				w = fa->src.w;
				do {
					*dst++ = *src++;
				} while(--w);

				src = (Pixel32 *)((char *)src + fa->src.modulo);
				dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
			} while(--h);

			rcDraw.top = (rcMask->top > rcMask->bottom)?rcMask->bottom:rcMask->top;
			rcDraw.bottom = (rcMask->top < rcMask->bottom)?rcMask->bottom:rcMask->top;
			rcDraw.left = (rcMask->left < rcMask->right)?rcMask->left:rcMask->right;
			rcDraw.right = (rcMask->left > rcMask->right)?rcMask->left:rcMask->right;


			xRectFill(fa,rcDraw.left,rcDraw.top,rcDraw.right - rcDraw.left,2,0xffffff);
			xRectFill(fa,rcDraw.left,rcDraw.top,2,rcDraw.bottom - rcDraw.top,0xffffff);
			xRectFill(fa,rcDraw.right,rcDraw.top,2,rcDraw.bottom - rcDraw.top,0xffffff);
			xRectFill(fa,rcDraw.left,rcDraw.bottom,rcDraw.right - rcDraw.left,2,0xffffff);


	} else {

		//--- Move src to dst ---

		h = fa->src.h;

		do {
			w = fa->src.w;
			do { *dst++ = *src++;} while(--w);

			src = (Pixel32 *)((char *)src + fa->src.modulo);
			dst = (Pixel32 *)((char *)dst + fa->dst.modulo);
		} while(--h);


		//--- If we have a file ---
		if(mfd->pgf) {

			lpbih = (LPBITMAPINFOHEADER) AVIStreamGetFrame(mfd->pgf, mfd->saveFrame);

			//--- if we have a Frame 
			if(lpbih) {

				//---- Make a vdub bitmap 
				mfd->pData = (LPBYTE)lpbih + lpbih->biSize;
				mfd->mySrc = &(mfd->mySrc->init(mfd->pData, lpbih));

				//--- If we are fit to frame
				if(mfd->inverse) {

					//--- If not 32 make it so --
					if(mfd->mySrc->depth !=32) {
						fa->src.BitBlt(0,0,(VBitmap *)mfd->mySrc,0,0,-1,-1);

						fa->dst.StretchBltNearestFast(rcMask->left,rcMask->top
							,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top
							,&(fa->src),0,0,fa->src.w,fa->src.h);

					//---- Was 32 bit so mySrc to dst
					} else {

						fa->dst.StretchBltNearestFast(rcMask->left,rcMask->top
							,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top
							,mfd->mySrc,0,0,mfd->mySrc->w,mfd->mySrc->h);
					}

				//---- No stretch just blit
				} else {
					fa->dst.BitBlt(rcMask->left,rcMask->top,(VBitmap *)mfd->mySrc
						,0,0,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top);
				}
			} //---- No frame ? don't do squat

		//--- No file fill black
		} else {
			fa->dst.RectFill(rcMask->left,rcMask->top
				,rcMask->right - rcMask->left,rcMask->bottom - rcMask->top,0);
		}


	}

    return 0;
}


//****************************************
// EndProc - Done Filtering 
//
// Good place to clean up 
//****************************************
int ezMaskEndProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;


    return 0;
}

void deinitProc(FilterActivation *fa, const FilterFunctions *ff) {
	    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if(mfd->pgf) AVIStreamGetFrameClose(mfd->pgf);	// close the stream
    if(mfd->pfile) AVIFileRelease(mfd->pfile);  // closes the file 
}


//****************************************
// ParamProc - Set the Frame size
//
// Good place to .....
//****************************************
long ezMaskParamProc(FilterActivation *fa, const FilterFunctions *ff) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	fa->dst.pitch = (fa->dst.w*4 + 7) & -8;

	mfd->dstW = fa->dst.w;
	mfd->dstH = fa->dst.h;


    return FILTERPARAM_SWAP_BUFFERS;
}


//****************************************
// ConfigDlgProc - Controls our configuration dialog
//
// This is where you need to do windows programing.
//****************************************
BOOL CALLBACK ConfigDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
    MyFilterData *mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

	static	char msgDraw[] = "Left click and drag in the preview window to draw the mask.";
	static	char msgMove[] = "Left click on the mask, and drag to move it in the preview window.";

	//

    switch(msg) {
        case WM_INITDIALOG:
            SetWindowLong(hdlg, DWL_USER, lParam);
            mfd = (MyFilterData *)lParam;

            CheckDlgButton(hdlg, IDC_MOVE, mfd->mode?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_DRAW, !(mfd->mode)?BST_CHECKED:BST_UNCHECKED);
            CheckDlgButton(hdlg, IDC_INV, mfd->inverse)?BST_CHECKED:BST_UNCHECKED;
			SetDlgItemText(hdlg, IDC_INFO, (mfd->mode)?msgMove:msgDraw);

			SetDlgItemText(hdlg, IDC_FILE, mfd->szAviPath);


			mfd->dlgHwnd = hdlg;
			
			if (mfd->ifp) {
				mfd->ifp->InitButton(GetDlgItem(hdlg, IDC_PREVIEW));
				mfd->ifp->Toggle(hdlg);
			}



            return TRUE;

        case WM_COMMAND:
            switch(LOWORD(wParam)) {
            case IDOK:
				mfd->mode = !!IsDlgButtonChecked(hdlg, IDC_MOVE);
                EndDialog(hdlg, 0);
                return TRUE;
            case IDCANCEL:
                EndDialog(hdlg, 1);
                return FALSE;
            case IDC_MOVE:
            case IDC_DRAW:
				mfd->mode = !!IsDlgButtonChecked(hdlg, IDC_MOVE);
				SetDlgItemText(hdlg, IDC_INFO, (mfd->mode)?msgMove:msgDraw);
				return TRUE;
            case IDC_INV:
				mfd->inverse = IsDlgButtonChecked(hdlg, IDC_INV);
				mfd->ifp->RedoFrame();
				return TRUE;
			case IDC_PREVIEW:
				if (mfd->ifp)
					mfd->ifp->Toggle(hdlg);
				return TRUE;

			case IDC_FILE_BROWSE:
				if (const char *fn = openFileDlg(hdlg, mfd->szAviPath)) {
					SetDlgItemText(hdlg, IDC_FILE, fn);
					strcpy(mfd->szAviPath, fn);
					testAVIFile(mfd->szAviPath, mfd);
					if (mfd->ifp) {mfd->ifp->RedoFrame();}
				}
				return TRUE;


            }
            break;
    }

    return FALSE;
}

//****************************************
// fPreviewButtonCallback - Vdub calls when preview is opened/closed
//
// Turn on/off capture of preview window mouse events
//****************************************

void *fPreviewButtonCallback(bool fNewState, void *pData) {

	MyFilterData *mfd = (MyFilterData *) pData;
	g_mfd = mfd;


	if(fNewState) {
		mfd->fpHwnd = FindWindow(0,"Filter Preview");

		//*** Install filter proc for preview windows messages ....
		mfd->fpWndProc = (WNDPROC) SetWindowLong(mfd->fpHwnd, GWL_WNDPROC, (LONG) nPreviewWndProc );

	} else {

		//*** restore proc for filter preview windows messages ....
		mfd->fpWndProc = (WNDPROC) SetWindowLong(mfd->fpHwnd, GWL_WNDPROC, (LONG) mfd->fpWndProc );
	}
	return FALSE;
}



//****************************************
// ConfigProc - Just before our dialog box is opened
//
// This is where you .....
//****************************************
int ezMaskConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	//---- Setup preview w/ callback on toggle
	mfd->ifp = fa->ifp;
	mfd->ifp->SetButtonCallback((FilterPreviewButtonCallback)fPreviewButtonCallback, mfd); 


    return DialogBoxParam(fa->filter->module->hInstModule,
            MAKEINTRESOURCE(IDD_FILTER_EZMASK), hwnd,
            (DLGPROC) ConfigDlgProc, (LPARAM)fa->filter_data);
}


//****************************************
// StringProc - Called to get onfiguration settings you
// see in the filters config list.
//
// Basically builds a string of the current settings.
//****************************************
void ezMaskStringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    //char *desc[50];
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    sprintf(str, " (%d,%d - %d,%d)",
        mfd->rcMask.left,
        mfd->rcMask.top,
        mfd->rcMask.right,
        mfd->rcMask.bottom);
}


//****************************************
// ScriptConfig - Sets your settings from a list
// of script arguments.
//****************************************
void ezMaskScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    mfd->rcMask.left    = argv[0].asInt();
    mfd->rcMask.top    = argv[1].asInt();
    mfd->rcMask.right    = argv[2].asInt();
    mfd->rcMask.bottom    = argv[3].asInt();
}

//****************************************
// FssProc - Saves your args as a string for scripts.
//****************************************
bool ezMaskFssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

    _snprintf(buf, buflen, "Config(%d,%d,%d,%d)",
        mfd->rcMask.left,
        mfd->rcMask.top,
        mfd->rcMask.right,
        mfd->rcMask.bottom);

    return true;
}


////////////////////////////////////////////
// Msg Proc to process filter preview windows messages
////////////////////////////////////////////

LRESULT CALLBACK nPreviewWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {

    POINTS ptLBD;
	POINT ptLBD2;
    RECT rcClient, *rcMask;                 // client area rectangle 
    POINT ptClientUL;              // client upper left corner 
    POINT ptClientLR;              // client lower right corner 


	switch(msg) {

        case WM_MOUSEMOVE: 
 
            // When moving the mouse, the user must hold down 
            // the left mouse button to draw lines. 
 
            if (wParam & MK_LBUTTON)  { 
				if(g_mfd->drawing == TRUE) {
					rcMask = (RECT *) &(g_mfd->rcMask);

					ptLBD = MAKEPOINTS(lParam); 
					ptLBD.y = ptLBD.y -4;
					ptLBD.x = ptLBD.x -4;

					if(g_mfd->mode == 0) {
						rcMask->bottom = ptLBD.y;
						rcMask->right = ptLBD.x;
					} else {
						OffsetRect(rcMask, ptLBD.x - g_mfd->ptsMove.x , ptLBD.y - g_mfd->ptsMove.y);
						g_mfd->ptsMove.x = ptLBD.x; g_mfd->ptsMove.y = ptLBD.y;
					}

					//**** The labels in the other dialog ****
					SetDlgItemInt(g_mfd->dlgHwnd, IDC_TOP, 
						(rcMask->top<rcMask->bottom)?max(rcMask->top,0):max(rcMask->bottom,0), FALSE);
					SetDlgItemInt(g_mfd->dlgHwnd, IDC_LEFT,
						(rcMask->left<rcMask->right)?max(rcMask->left,0):max(rcMask->right,0), FALSE);
					SetDlgItemInt(g_mfd->dlgHwnd, IDC_RIGHT, 
						(rcMask->left<rcMask->right)?min(rcMask->right,g_mfd->dstW):min(rcMask->left,g_mfd->dstW), FALSE);
					SetDlgItemInt(g_mfd->dlgHwnd, IDC_BOTTOM, 
						(rcMask->top<rcMask->bottom)?min(rcMask->bottom,g_mfd->dstH):min(rcMask->top,g_mfd->dstH), FALSE);

					g_mfd->ifp->RedoFrame();
				}
			}
			break;

       case WM_LBUTTONDOWN: 
 
		ptLBD = MAKEPOINTS(lParam); 
		POINTSTOPOINT(ptLBD2,ptLBD);

		if(g_mfd->mode==1) { //*** Move

			//** Did you click in the rectangle ?
			rcMask = (RECT *) &(g_mfd->rcMask);
			if(PtInRect(rcMask, ptLBD2) ){
				g_mfd->drawing = TRUE;
 
				GetClientRect(hWnd, &rcClient); 
				ptClientUL.x = rcClient.left + 4; 
				ptClientUL.y = rcClient.top + 4; 
				ptClientLR.x = ptClientUL.x + g_mfd->dstW; 
				ptClientLR.y = ptClientUL.y + g_mfd->dstH; 

				ClientToScreen(hWnd, &ptClientUL); 
				ClientToScreen(hWnd, &ptClientLR); 
 
				SetRect(&rcClient, ptClientUL.x, ptClientUL.y, ptClientLR.x, ptClientLR.y); 
				ClipCursor(&rcClient); 

				g_mfd->ptsMove.x = ptLBD.x; g_mfd->ptsMove.y = ptLBD.y;
			   
			}


		} else { //--- Draw Mode
		
			//**** If click in client area start draw 
			if(ptLBD.x-3 > 0 
				&& ptLBD.x-3 < g_mfd->dstW 
				&& ptLBD.y-3 > 0 
				&& ptLBD.y-3 < g_mfd->dstH) {

				g_mfd->drawing = TRUE;

				GetClientRect(hWnd, &rcClient); 
				ptClientUL.x = rcClient.left + 4; 
				ptClientUL.y = rcClient.top + 4; 
				ptClientLR.x = ptClientUL.x + g_mfd->dstW; 
				ptClientLR.y = ptClientUL.y + g_mfd->dstH; 

				ClientToScreen(hWnd, &ptClientUL); 
				ClientToScreen(hWnd, &ptClientLR); 
 
				SetRect(&rcClient, ptClientUL.x, ptClientUL.y, ptClientLR.x, ptClientLR.y); 
				ClipCursor(&rcClient); 


				rcMask = (RECT *) &(g_mfd->rcMask);

				//**** Set Rect to 1x1 at click point
				SetRect(rcMask, ptLBD.x -4, ptLBD.y -4, ptLBD.x -3, ptLBD.y -3);
			}
		}
		//g_mfd->frame = g_mfd->saveFrame; 
			
		break;

       case WM_LBUTTONUP: 


		   if(g_mfd->drawing) {

				rcMask = (RECT *) &(g_mfd->rcMask);
			   if(g_mfd->mode == 0) {

					rcClient.bottom = (rcMask->top < rcMask->bottom)?rcMask->bottom:rcMask->top;
					rcClient.top = (rcMask->top > rcMask->bottom)?rcMask->bottom:rcMask->top;
					rcClient.right = (rcMask->left > rcMask->right)?rcMask->left:rcMask->right;
					rcClient.left = (rcMask->left < rcMask->right)?rcMask->left:rcMask->right;

					//*** Make sure UL is before LR
					rcMask->bottom = rcClient.bottom;
					rcMask->top = rcClient.top;
					rcMask->right = min(rcClient.right,g_mfd->dstW - 1);
					rcMask->left = max(rcClient.left,0);

			   } else {
					//*** Make sure all points are on screen
					rcMask->bottom = min(rcMask->bottom, g_mfd->dstH - 1);
					rcMask->top = max(rcMask->top,0);
					rcMask->right = min(rcMask->right,g_mfd->dstW - 1);
					rcMask->left = max(rcMask->left,0);
			   }
				ClipCursor(NULL); 
				g_mfd->drawing = FALSE;
				g_mfd->ifp->RedoFrame();
		   }

			//default:
	}

	return CallWindowProc( g_mfd->fpWndProc, hWnd, msg, wParam, lParam);
//	return CallWindowProc((DLGPROC) g_mfd->fpWndProc, hWnd, msg, wParam, lParam);


}


//*********************************

void xRectFill(const FilterActivation *fa, PixCoord x, PixCoord y, PixDim dx, PixDim dy, Pixel32 c) {


	// Do the blit

	Pixel32 *dst = (Pixel32 *)fa->dst.data;
	int w = fa->dst.w;
	int h = fa->dst.h;
	int m = fa->dst.modulo;
	PixOffset pitch = fa->dst.pitch;



	if (dx == -1) dx = w;
	if (dy == -1) dy = h;

	// clip to destination bitmap

	if (x < 0) { dx+=x; x=0; }
	if (y < 0) { dy+=y; y=0; }
	if (x+dx > w) dx=w-x;
	if (y+dy > h) dy=h-y;

	// anything left to fill?

	if (dx<=0 || dy<=0) return;

	// compute coordinates

	dst = fa->dst.Address32(x, y+dy-1);

	// do the fill

	do {
		PixDim dxt = dx;
		Pixel32 *dst2 = dst;

		do {
			*dst2++ = c;
		} while(--dxt);

		dst = (Pixel32 *)((char *)dst + pitch);
	} while(--dy);

return;

}

static const char *openFileDlg(HWND hwnd, const char *oldfn) {
	OPENFILENAME ofn;
	static char szFile[MAX_PATH];
	char szFileTitle[MAX_PATH];

	///////////////

	if (oldfn)
		strcpy(szFile, oldfn);

	szFileTitle[0]=0;

	ofn.lStructSize			= sizeof (OPENFILENAME) ;
	ofn.hwndOwner			= hwnd;
	ofn.lpstrFilter			= "AVI file (*.avi,*.avs)\0*.avi;*.avs\0All files (*.*)\0*.*\0";
	ofn.lpstrCustomFilter	= NULL;
	ofn.nFilterIndex		= 1;
	ofn.lpstrFile			= szFile;
	ofn.nMaxFile			= sizeof szFile;
	ofn.lpstrFileTitle		= szFileTitle;
	ofn.nMaxFileTitle		= sizeof szFileTitle;
	ofn.lpstrInitialDir		= NULL;
	ofn.lpstrTitle			= "Select AVI";
	ofn.Flags				= OFN_EXPLORER | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_ENABLESIZING;
	ofn.lpstrDefExt			= NULL;

	if (GetOpenFileName(&ofn))
		return szFile;

	return NULL;
}

void testAVIFile(char * filename, MyFilterData *mfd) {


  LONG hr; //--- Return code from file open.  0 is good 
  AVIFILEINFO pfi; //--- Structure to hold File Info 
  AVISTREAMINFO psi; //--- Structure to hold Stream Info (where the fccHandler is) 
  HRESULT hresult; 
  long bmiSize;
  BITMAPINFOHEADER *bmih;
  char buf[20];
  
     
 

//---- Open the file.  If you don't get a zero back, Tell avs there  was an error 
  hr = AVIFileOpen(&mfd->pfile, filename, OF_READ, NULL); 
  if (hr != 0){ 
      return; 
  } 
  

//---- Get the File Info into our local structure so we can look 
   AVIFileInfo(mfd->pfile, &pfi, sizeof(pfi)); 


    //    if (AVIFileGetStream(pfile, &ppavi, NULL, streamtypeVIDEO) != AVIERR_OK) 
   AVIFileGetStream(mfd->pfile, &mfd->ppavi, streamtypeVIDEO, 0);
    //    break; 
  
      if (mfd->ppavi != NULL)  { 
         hresult = AVIStreamInfo(mfd->ppavi, &psi, sizeof(psi)); 

         sprintf(buf,"%c%c%c%c",psi.fccHandler, psi.fccHandler>>8, psi.fccHandler>>16, psi.fccHandler>>24); 
         sprintf(buf,"%c%c%c%c", psi.fccType , psi.fccType>>8, psi.fccType>>16, psi.fccType>>24); 

		 if(AVIStreamReadFormat(mfd->ppavi,AVIStreamStart(mfd->ppavi),NULL, &bmiSize))
			 return;

		 LPBYTE pChunk = new BYTE[bmiSize];
		 if(!pChunk)
			 return; //--- Allocation Error

		 if(AVIStreamReadFormat(mfd->ppavi,AVIStreamStart(mfd->ppavi),pChunk, &bmiSize))
			 return;

		 bmih = (LPBITMAPINFOHEADER) &pChunk;
  		 mfd->pgf = AVIStreamGetFrameOpen(mfd->ppavi,NULL);

	  }


} 
